﻿/// <reference path="../libs/angular/angular.js" />
/// <reference path="../libs/jquery/jquery-1.10.1.js" />
'use strict';

/* Controllers */

var cont = angular.module('mobilize')
.controller('content', ['$scope', '$poi', '$routeParams', '$timeout',
    function ($scope, $poi, $routeParams, $timeout) {

        $scope.page = $scope.getPage($routeParams.content);

        $scope.content = "/app/partials/content/" + $scope.page.key + ".html";

        $scope.scopeTitle = $scope.page.title;
        $scope.poitype = $scope.page.poiKey;
        $scope.location = "";



        $scope.$watch('poitype', function () {
            if ($scope.poitype)
                $scope.poiEnabled = true;
            else
                $scope.poiEnabled = false;
            $scope.search();
        });

        $scope.$watch('location', function () {
            $scope.search();
        });

        $scope.pois = [];

        $scope.loading = true;

        $scope.poiEnabled = false;
        $scope.contentVisible = true;
        $scope.mapVisible = false;
        $scope.listVisible = false;
        $scope.searchBoxVisible = false;
        $scope.searchBoxOpen = false;

        $scope.showContent = function () {
            $scope.contentVisible = true;
            $scope.mapVisible = false;
            $scope.listVisible = false;
        };

        $scope.showMap = function () {
            $scope.contentVisible = false;
            $scope.mapVisible = true;
            $scope.listVisible = false;
        };

        $scope.showServices = function () {
            $scope.contentVisible = false;
            $scope.mapVisible = true;
            $scope.listVisible = true;
        };

        $scope.showList = function () {
            $scope.contentVisible = false;
            $scope.mapVisible = false;
            $scope.listVisible = true;
            scrollSelectedPoiIntoView();
        };

        $scope.openSearchBox = function () {
            $scope.searchBoxOpen = true;
        };
        $scope.selectedPoi = {};

        function scrollSelectedPoiIntoView() {
            var attempt = 0;
            function run() {
                attempt++;
                if (attempt < 10) {
                    $timeout(function () {
                        var elm = $('#' + $scope.selectedPoi.id);
                        if (elm.length > 0 && elm.position().top > 0) {
                            $('html, body').scrollTop(elm.position().top);
                        } else {
                            run();
                        }
                    }, 1);
                }
            }

            if ($scope.selectedPoi.id){
                
                run();
            }
        }

        $scope.toggleSelectedPoi = function (poi) {
            if (poi == $scope.selectedPoi)
                $scope.selectedPoi = {};
            else
                $scope.selectedPoi = poi;

            $scope.selectedmarker = null;
            for (var i in $scope.mapmarkers) {
                if ($scope.mapmarkers[i].id == poi.id)
                    $scope.selectedmarker = $scope.mapmarkers[i];
            }

            scrollSelectedPoiIntoView();           
        }
        var srchLoop = 0;

        $scope.mapcenter = {};
        $scope.mapmarkers = [];
        $scope.selectedmarker = null;

        $scope.$on('leafletDirectiveMarkersClick', function (e, idx) {
            if (!$scope.isTablet) {
                e.preventDefault();
                $scope.showList();
            }
            $scope.toggleSelectedPoi($scope.pois[idx]);            
        });

        $scope.search = function () {
            $scope.loading = true;
            if ($scope.poitype) {
                srchLoop++;
                $poi.search($scope.poitype, $scope.location)
                    .then(function (data) {
                        srchLoop = 0;

                        $scope.mapcenter = {
                            lat: data.data.location.lat,
                            lng: data.data.location.lon,
                            zoom:12
                        };

                        while ($scope.pois.length > 0)
                            $scope.pois.pop();
                        while ($scope.mapmarkers.length > 0)
                            $scope.mapmarkers.pop();


                        var selectedFixed = false;
                        for (var i in data.data.poi) {
                            var poi = data.data.poi[i];
                            if ($scope.selectedPoi.id == poi.id) {
                                $scope.selectedPoi = poi;
                                selectedFixed = true;
                            }
                            $scope.pois.push(poi);

                            $scope.mapmarkers.push({
                                lat: poi.lat,
                                lng: poi.lon,
                                focus: true,
                                draggable: false,
                                id : poi.id
                            });
                            $scope.mapcenter.lat = (poi.lat + $scope.mapcenter.lat) / 2;
                            $scope.mapcenter.lng = (poi.lon + $scope.mapcenter.lng) / 2;
                        }
                        if (!selectedFixed)
                            $scope.selectedPoi = {};
                        $scope.loading = false;
                    },
                    function (error) {
                        if (srchLoop < 10)
                            $scope.search();
                        else {
                            alert('failing to search');
                            $scope.poiEnabled = false;
                        }
                    });
            }
        };
}])