﻿/// <reference path="../libs/angular/angular.js" />



angular.module('mobilize.services')
.factory('$poi', ['$q', '$rootScope', '$geoLocation', '$http', "$timeout", function ($q, $rootScope, $geoLocation, $http, $timeout) {

    var pastFailure = false;
    var defaultLocation = "nottingham";
    var geoLocationTimeout = 5; //this value is in seconds, and is converted to milliseconds in the line
    function getLocationQueryStringParam(locationStringIfKnown) {
        var d = $q.defer();

        if (locationStringIfKnown) {
            d.resolve({
                loc: locationStringIfKnown
            });
        } else {
            if (pastFailure) {
                d.resolve({
                    loc: defaultLocation
                });
            }

            var timedOut = false;
            var resolved = false;
            //this is a watch dog timeout
            $timeout(function () {
                if (!resolved) {
                    timedOut = true;
                    resolved = true;
                    pastFailure = true;
                    d.resolve({
                        loc: defaultLocation
                    });
                }
            }, geoLocationTimeout*1000);

            $geoLocation.location().then(function (data) {
                d.resolve({
                    lon: data.longitude,
                    lat: data.latitude
                });
                resolved = true;
                pastFailure = false;
            }, function (reason) {
                if (timedOut) {
                    //rejected??
                    d.resolve({
                        loc: defaultLocation
                    });
                    resolved = true;
                }
            });
        }

        return d.promise;
    }

    var searchUrl = "http://poi.nationalservers.co.uk/v1/search";
    var commonParams = {
        format: 'json'
    };
    var userParams = {
        key: '',
        limit: 15
    };

    var cache = {};

    return {
        setup: function (params) {
            angular.extend(userParams, params)
        },

        search: function (type, location) {

            var d = $q.defer();

            getLocationQueryStringParam(location)
                .then(function (v) {

                    var qsParams = angular.extend({ type: type }, commonParams, userParams, v);
                    var url = searchUrl;

                    qsParams.callback = 'JSON_CALLBACK';
                    var joiner = '?';
                    if (url.indexOf('?') > 0)
                        joiner = '&';

                    for (var i in qsParams) {
                        url += joiner + i + '=' + encodeURIComponent(qsParams[i]);
                        joiner = '&';
                    }

                    if (cache[url]) {
                        var sendResponse = function () {
                            $rootScope.$apply(function () {
                                if (cache[url] != "loading") {
                                    d.resolve(cache[url]);
                                } else {
                                    setTimeout(sendResponse, 100);
                                }
                            });
                        }
                        setTimeout(sendResponse, 0);
                    } else {
                        cache[url] = "loading";
                        $http.jsonp(url, { cache: true }).then(function (data) {
                            cache[url] = data.data;
                            d.resolve(data.data);
                        }, function (reason) {
                            delete cache[url];
                            d.reject(reason);
                        });
                    }
                },
                function (reason) {
                    d.reject(reason);
                });

            return d.promise;
        }
    };
} ]);


